/*
 *
 *  Library for simple D-Bus integration with GLib
 *
 *  Copyright (C) 2007-2008  Intel Corporation. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/signal.h>

#include "gdbus.h"

static DBusMessage *test_method(DBusConnection *conn,
					DBusMessage *msg, void *data)
{
	const char *text = "Hello World";
	DBusMessage *reply;

	reply = dbus_message_new_method_return(msg);
	if (!reply)
		return NULL;

	dbus_message_append_args(reply,
			DBUS_TYPE_STRING, &text, DBUS_TYPE_INVALID);

	return reply;
}

static DBusMessage *test_error(DBusConnection *conn,
					DBusMessage *msg, void *data)
{
	return g_dbus_create_error(msg, "org.example.error.Invalid", NULL);
}

static GDBusMethodTable methods[] = {
	{ "Test",  "", "s", test_method },
	{ "Error", "", "",  test_error  },
	{ },
};

static GDBusSignalTable signals[] = {
	{ "Tested", "" },
	{ },
};

static GMainLoop *main_loop = NULL;

static void sig_term(int sig)
{
	g_main_loop_quit(main_loop);
}

int main(int argc, char *argv[])
{
	DBusConnection *conn;
	DBusError err;
	struct sigaction sa;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = sig_term;
	sigaction(SIGINT, &sa, NULL);
	sigaction(SIGTERM, &sa, NULL);

	sa.sa_handler = SIG_IGN;
	sigaction(SIGCHLD, &sa, NULL);
	sigaction(SIGPIPE, &sa, NULL);

	main_loop = g_main_loop_new(NULL, FALSE);

	dbus_error_init(&err);

	conn = g_dbus_setup_bus(DBUS_BUS_SESSION, "org.example", &err);
	if (conn == NULL) {
		if (dbus_error_is_set(&err) == TRUE) {
			fprintf(stderr, "%s\n", err.message);
			dbus_error_free(&err);
		} else
			fprintf(stderr, "Can't register with session bus\n");
		exit(1);
	}

	g_dbus_register_object(conn, "/test");

	g_dbus_register_interface(conn, "/test", "org.example.Test",
					methods, signals, NULL, NULL, NULL);

	g_main_loop_run(main_loop);

	g_dbus_unregister_interface(conn, "/test", "org.example.Test");

	g_dbus_unregister_object(conn, "/test");

	g_dbus_cleanup_connection(conn);

	g_main_loop_unref(main_loop);

	return 0;
}
